VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DBTurbo2DEngine"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

'These Are Instances of all the classes
Public DBMath As New DBMath
Public DBTexture As New DBTexture
Public DBOverlay As New DBOverlay
Public DBFPS As New DBFPS
Public DBText As New DBText
Public DBMap As New DBMap
Public DBSubMap As New DBSubMap
Public DBSprite As New DBSprite
Public DBAutoMove As New DBAutoMove
Public DBKeyInput As New DBKeyInput
Public DBJoyStick As New DBJoyStick
Public DBMouse As New DBMouse
Public DBMusic As New DBMusic
Public DBSound As New DBSound
Public DBMapData As New DBMapData
Public DBCounter As New DBCounter
Public DBResource As New DBResource

Public Sub InitializeDisplay(hwnd&, dWindowed As Boolean, Optional dWidth& = 320, Optional dHeight& = 240, Optional dAdapter As Long = 0, Optional DevType As CONST_D3DDEVTYPE)
    On Error GoTo NotInitialized
    
'Variables for displayMode
    Dim dParams As D3DPRESENT_PARAMETERS
    Dim DispMode As D3DDISPLAYMODE
    
'Set up Screen Variables
    DisplayWidth = dWidth
    DisplayHeight = dHeight
    
'Initialize Direct3d
    
    Set DirectX = New DirectX8
    Set Direct3D = DirectX.Direct3DCreate
    Set D3DX = New D3DX8

'Get Current DisplayMode
    Direct3D.GetAdapterDisplayMode dAdapter, DispMode

'Set the Device to Hardware if empty
    If DevType = 0 Or DevType = 3 Then DevType = D3DDEVTYPE_HAL

'Set Up Parameters for displaymode
    If dWindowed = True Then
        
        With dParams
        .BackBufferFormat = DispMode.Format
        .BackBufferWidth = dWidth
        .BackBufferHeight = dHeight
        .EnableAutoDepthStencil = False
        .Windowed = 1
        .SwapEffect = D3DSWAPEFFECT_COPY
        End With
    Else:
        With dParams
        .BackBufferFormat = DispMode.Format
        .EnableAutoDepthStencil = False
        .BackBufferWidth = dWidth
        .BackBufferHeight = dHeight
        .Windowed = 0
        .SwapEffect = D3DSWAPEFFECT_FLIP
        End With
    End If

UParams = dParams

'Set up DisplayMode
    Set Direct3DDevice = Direct3D.CreateDevice(dAdapter, DevType, hwnd, D3DCREATE_HARDWARE_VERTEXPROCESSING, dParams)
'Set Up Vertex Format
    Direct3DDevice.SetVertexShader FVF_VertexType
    
'Turn Off Lighting
Direct3DDevice.SetRenderState D3DRS_LIGHTING, False
Direct3DDevice.SetRenderState D3DRS_SPECULARENABLE, True

'Set Render State
    Direct3DDevice.SetRenderState D3DRS_SRCBLEND, D3DBLEND_SRCALPHA
    Direct3DDevice.SetRenderState D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA
    Direct3DDevice.SetRenderState D3DRS_ALPHABLENDENABLE, True
    
'Set Up Texture Stages
    Direct3DDevice.SetTextureStageState 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE
    Direct3DDevice.SetTextureStageState 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE
    Direct3DDevice.SetTextureStageState 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE
    DaBoodaDisplayOn = True
    
'Set Up dummy viewport and reg viewport
    With DViewPort
        .MaxZ = 0
        .MinZ = 0
        .X = 0
        .Y = 0
        .Width = dWidth
        .Height = dHeight
    End With
    With ViewPort
        .MaxZ = 0
        .MinZ = 0
        .X = 0
        .Y = 0
        .Width = dWidth
        .Height = dHeight
    End With

'set up a default map view
    With MapView
        .Top = 0
        .Left = 0
        .Right = dWidth
        .Bottom = dHeight
    End With
    MapClip = 0
    
'create the sprite for overlay
    Set oSprite = D3DX.CreateSprite(Direct3DDevice)
    
'Initialize the Classes
Set DBMath = New DBMath
Set DBTexture = New DBTexture
Set DBOverlay = New DBOverlay
Set DBFPS = New DBFPS
Set DBText = New DBText
Set DBMap = New DBMap
Set DBSubMap = New DBSubMap
Set DBSprite = New DBSprite
Set DBAutoMove = New DBAutoMove
Set DBKeyInput = New DBKeyInput
Set DBJoyStick = New DBJoyStick
Set DBMouse = New DBMouse
Set DBMusic = New DBMusic
Set DBSound = New DBSound
Set DBMapData = New DBMapData
Set DBCounter = New DBCounter
Set DBResource = New DBResource

    Exit Sub
NotInitialized:
    DaBoodaDisplayOn = False
End Sub

'Set texture stage
Public Sub SetTextureStage(TType As CONST_D3DTEXTURESTAGESTATETYPE, TState As Long)

Direct3DDevice.SetTextureStageState 0, TType, TState

End Sub

'Set RenderState
Public Sub SetRenderState(RType As CONST_D3DRENDERSTATETYPE, Value As Long)

Direct3DDevice.SetRenderState RType, Value

End Sub

'sub to set up viewport, not to be confused with mapview
Public Sub SetUpViewPort(X As Long, Y As Long, Width As Long, Height As Long)
    With ViewPort
        .MaxZ = 0
        .MinZ = 0
        .X = X
        .Y = Y
        .Width = Width
        .Height = Height
    End With
    
    Direct3DDevice.SetViewport ViewPort
    
End Sub

'Subs to set colors
Public Sub SetBackColorRed(Value As Integer)
    DisplayColorInfo.Red = Value
    DisplayColor = D3DColorARGB(255, DisplayColorInfo.Red, DisplayColorInfo.Green, DisplayColorInfo.Blue)
End Sub
Public Sub SetBackColorGreen(Value As Integer)
    DisplayColorInfo.Green = Value
    DisplayColor = D3DColorARGB(255, DisplayColorInfo.Red, DisplayColorInfo.Green, DisplayColorInfo.Blue)
End Sub
Public Sub SetBackColorBlue(Value As Integer)
    DisplayColorInfo.Blue = Value
    DisplayColor = D3DColorARGB(255, DisplayColorInfo.Red, DisplayColorInfo.Green, DisplayColorInfo.Blue)
End Sub
Public Function GetBackColorRed() As Integer
    GetBackColorRed = DisplayColorInfo.Red
End Function
Public Function GetBackColorGreen() As Integer
    GetBackColorGreen = DisplayColorInfo.Green
End Function
Public Function GetBackColorBlue() As Integer
    GetBackColorBlue = DisplayColorInfo.Blue
End Function

'this is for windowed mode and to check focus and minimized
Public Sub SetWindowedObject(Form As Object)
    Set WindowObject = Form
    WindowCheck = True
End Sub

'This sets up the view rect, where the maps and sprites are actually drawn
Public Sub SetUpMapView(mLeft As Single, mTop As Single, mRight As Single, mBottom As Single, mClip As Single)
'place Values into mapview rect
With MapView
    .Left = mLeft
    .Right = mRight
    .Top = mTop
    .Bottom = mBottom
End With
    MapClip = mClip
End Sub

'Subs to Set LevelMax
Public Sub SetMaxLevel(Value As Integer)
    MaxLevel = Value
End Sub
Public Function GetMaxLevel() As Integer
    GetMaxLevel = MaxLevel
End Function

Public Function GetDisplayOn() As Boolean
    GetDisplayOn = DaBoodaDisplayOn
End Function

Public Function GetSoundOn() As Boolean
    GetSoundOn = DaBoodaSoundOn
End Function

Public Function GetMusicOn() As Boolean
    GetMusicOn = DaBoodaMusicOn
End Function

Public Function GetKeyInputOn() As Boolean
    GetKeyInputOn = DaBoodaKeyInputOn
End Function

Public Function GetMouseOn() As Boolean
    GetMouseOn = DaBoodaMouseOn
End Function

Public Function GetJoyStickOn() As Boolean
    GetJoyStickOn = DaBoodaJoyStickOn
End Function

Public Function GetFontOn() As Boolean
    GetFontOn = DaBoodaFontOn
End Function

'This Sub is important
'It takes a pixel value for texture and returns it into a value
'from 0 to 1
Private Function TextureValue(Total As Single, Current As Single) As Single
Dim PixelTex As Single
PixelTex = 1 / (Total)
TextureValue = PixelTex * (Current)
End Function

'This Function Makes the Strips
Private Function MakeStrip(X As Single, Y As Single, Z As Single, rhw As Single, Color As Long, specular As Long, tu As Single, tv As Single) As TLVertex
MakeStrip.X = X
MakeStrip.Y = Y
MakeStrip.Z = Z
MakeStrip.rhw = rhw
MakeStrip.Color = Color
MakeStrip.specular = specular
MakeStrip.tu = tu
MakeStrip.tv = tv

End Function

'Sub to render Screen
'Probably the mose important sub in engine
'next to the initialization sub
Public Sub Render()
'Set up Variable for Loop
    Dim Count As Integer
    Dim CountIndex As Integer
    
'do events........keeps program from going into infinite loop
    DoEvents

'Check to see if device is lost, and go into a loop to prevent error
'This is a check to see if the user hit alt-tab
If Direct3DDevice.TestCooperativeLevel <> D3D_OK Then
    'Pause the music if running
        If DaBoodaMusicOn = True Then
            DBMusic.Pause
        End If
    'Do a loop while device is lost
        Do While Direct3DDevice.TestCooperativeLevel = D3DERR_DEVICELOST
            'Do events to check for Window resized
            DoEvents
        Loop
    'unaquire mouse
        If DaBoodaMouseOn = True Then
            mDirectInputDevice.Unacquire
        End If
    'clear font from memory
        If DaBoodaFontOn = True Then
            For Count = 1 To 16
                If FontSet(Count) = True Then
                    ScreenFont(Count).OnLostDevice
                End If
            Next Count
        End If
    'clear the sprite
        oSprite.OnLostDevice
    'reset device
        Direct3DDevice.Reset UParams
    'restore font
        If DaBoodaFontOn = True Then
            For Count = 1 To 16
                If FontSet(Count) = True Then
                    ScreenFont(Count).OnResetDevice
                End If
            Next Count
        End If
    'restore the sprite
        oSprite.OnResetDevice
    'Reload Vertex Data
        ReloadVertexData
    'clear the screen to prevent flicker
        ClearScreen
    'UnPause the Music
        If DaBoodaMusicOn = True Then
            DBMusic.UnPause
        End If
    'aquire mouse
        If DaBoodaMouseOn = True Then
            mDirectInputDevice.Acquire
        End If
'exit if
End If
            
'do the window check if windowed is on
    If WindowCheck = True Then CheckWindowed
    
'poll objects if needed
If DaBoodaMouseOn = True Then
    DBMouse.PollMouse
End If
If DaBoodaJoyStickOn = True Then
    DBJoyStick.PollJoyStick
End If
If DaBoodaKeyInputOn = True Then
    DBKeyInput.PollKeyBoard
End If

'Do Screen Limits
    If AutoMove.On = True Then
        CheckAutoMoveSprite
        UpDateMapToLimit
    End If

    CheckMapLimit
    CheckSpritePosition
    CheckSpriteReferences
    
'update the counters if needed
    If CounterCount <> 0 Then UpdateCounter

'clear screen
    Direct3DDevice.Clear 0, ByVal 0, D3DCLEAR_TARGET, DisplayColor, 1#, 0

'Begin scene
    Direct3DDevice.BeginScene

'Start Loop
    For Count = 0 To MaxLevel
'The Graphics
    'Do map
    For CountIndex = 1 To 8
        If Map(CountIndex).ZOrder = Count And Map(CountIndex).Visible = True Then
            RenderMap CountIndex
        End If
    Next CountIndex
    'Do Overlays
If OverlayCount > 0 Then
    For CountIndex = 1 To OverlayCount
        If Overlay(CountIndex).ZOrder = Count Then
            RenderOverlay CountIndex
        End If
    Next CountIndex
End If
    'do Sprites
If SpriteCount > 0 Then
    For CountIndex = 1 To SpriteCount
        If Sprite(CountIndex).AutoCounterInc = True And Sprite(CountIndex).Void = False And Sprite(CountIndex).ZOrder = Count Then
            Sprite(CountIndex).Counter = Sprite(CountIndex).Counter + Sprite(CountIndex).CounterInc
        End If
        If Sprite(CountIndex).ZOrder = Count Then
            RenderSprite CountIndex
        End If
    Next CountIndex
End If
    'Do Text
If TextCount > 0 Then
    For CountIndex = 1 To TextCount
        If TextEX(CountIndex).ZOrder = Count Then
            RenderText CountIndex
        End If
    Next CountIndex
End If
'Increment loop
    Next Count
    
'End Scene
    Direct3DDevice.EndScene

'DoDelay
'do frame delay, to reach FrameRate
    DBFPS.UpdateFPS
    
'Present to display
    Direct3DDevice.Present ByVal 0, ByVal 0, 0, ByVal 0
    
'CheckMusic
If MusicRepeat = True Then
     DBMusic.CheckMusicRepeat
End If

End Sub

'This is the RenderMap Sub
Private Sub RenderMap(Index As Variant)

'Variables for sub
    Dim X1 As Integer, X2 As Single, Y1 As Integer, Y2 As Single     'These are for positions
    Dim TopX As Single, TopY As Single                             'These are the top map positions
    Dim XCount As Integer, YCount As Integer
    Dim XLayer As Integer, YLayer As Integer
    Dim XStart As Single, YStart As Single
    Dim Num As Single
    
    Dim SubMapWidth As Single
    Dim SubMapHeight As Single
    Dim TMap As MapInfo
    Dim TSMap As SubMapInfo
    
    Dim TSize As Single
    
'Put map into Variable
    TMap = Map(Index)
    
'Get MapCounts...How many submaps are viewed on screen
Dim MapXCount As Single, MapYCount As Single
MapXCount = MapView.Right - MapView.Left
MapYCount = MapView.Bottom - MapView.Top
MapXCount = Int(MapXCount / TMap.SubMapWidth)
MapYCount = Int(MapYCount / TMap.SubMapHeight)

If (MapView.Right - MapView.Left) - (MapXCount * TMap.SubMapWidth) > 0 Then MapXCount = MapXCount + 1
If (MapView.Bottom - MapView.Top) - (MapYCount * TMap.SubMapHeight) > 0 Then MapYCount = MapYCount + 1

MapXCount = MapXCount + 1
MapYCount = MapYCount + 1

'Find StartPosition
    If TMap.XPos > 0 Then
        X1 = Int(TMap.XPos / TMap.SubMapWidth) + 1
        XLayer = 1 - (X1)
        XStart = TMap.XPos - (TMap.SubMapWidth * X1)
    End If
    If TMap.YPos > 0 Then
        Y1 = Int(TMap.YPos / TMap.SubMapHeight) + 1
        YLayer = 1 - (Y1)
        YStart = TMap.YPos - (TMap.SubMapHeight * Y1)
    End If
    If TMap.XPos < 0 Then
        X1 = Int(Abs(TMap.XPos) / TMap.SubMapWidth)
        XLayer = 1 + X1
        XStart = TMap.XPos + (TMap.SubMapWidth * X1)
    End If
    If TMap.YPos < 0 Then
        Y1 = Int(Abs(TMap.YPos) / TMap.SubMapHeight)
        YLayer = 1 + Y1
        YStart = TMap.YPos + (TMap.SubMapHeight * Y1)
    End If
    If TMap.XPos = 0 Then
        XLayer = 1
        XStart = 0
    End If
    If TMap.YPos = 0 Then
        YLayer = 1
        YStart = 0
    End If
    
'Offset these by mapview Values
            XStart = XStart + MapView.Left
            YStart = YStart + MapView.Top

'Loop through all Maps
    For XCount = 1 To MapXCount
        For YCount = 1 To MapYCount
        
'Place position into temp variables
            X1 = XStart + (TMap.SubMapWidth * (XCount - 1))
            Y1 = YStart + (TMap.SubMapHeight * (YCount - 1))

'Set x and y into different variables
    TopX = X1: TopY = Y1
    
'SetLayer Indexes
    X1 = XLayer + (XCount - 1)
    Y1 = YLayer + (YCount - 1)
    If X1 < 1 And TMap.Looping = False Then GoTo DontDraw
    If X1 > TMap.XCount And TMap.Looping = False Then GoTo DontDraw
    If Y1 < 1 And TMap.Looping = False Then GoTo DontDraw
    If Y1 > TMap.YCount And TMap.Looping = False Then GoTo DontDraw
    
    If X1 < 1 Then X1 = TMap.XCount - Abs(X1)
    If X1 > TMap.XCount Then X1 = (X1 - TMap.XCount)
    If Y1 < 1 Then Y1 = TMap.YCount - Abs(Y1)
    If Y1 > TMap.YCount Then Y1 = (Y1 - TMap.YCount)
    
    TSMap = SubMap(Index, X1, Y1)
    If TSMap.Visible = False Then GoTo DontDraw
    
'Set up Strip
            TextureStrip(0) = MakeStrip(TopX, TopY, 0, 1, TSMap.C1, TSMap.S1, TMap.Tx1, TMap.Ty1)
            TextureStrip(1) = MakeStrip(TopX + TMap.SubMapWidth, TopY, 0, 1, TSMap.C2, TSMap.S2, TMap.Tx2, TMap.Ty1)
            TextureStrip(2) = MakeStrip(TopX, TopY + TMap.SubMapHeight, 0, 1, TSMap.C3, TSMap.S3, TMap.Tx1, TMap.Ty2)
            TextureStrip(3) = MakeStrip(TopX + TMap.SubMapWidth, TopY + TMap.SubMapHeight, 0, 1, TSMap.C4, TSMap.S4, TMap.Tx2, TMap.Ty2)

'Set texture
            Direct3DDevice.SetTexture 0, Texture(TSMap.TextureRef)
            
'DrawStrip
            Direct3DDevice.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, TextureStrip(0), Len(TextureStrip(0))

'A goto point to goto if layer isn't Drawn
DontDraw:
        Next YCount
    Next XCount

End Sub

Private Sub RenderOverlay(Order As Integer)
Dim tOverlay As OverlayInfo
tOverlay = Overlay(Order)

If tOverlay.Visible = False Or tOverlay.TextureIndex = 0 Then GoTo DontDraw

Dim X1 As Single, Y1 As Single
Dim X2 As Single, Y2 As Single
Dim X3 As Single, Y3 As Single
Dim X4 As Single, Y4 As Single
Dim Angle As Single

tOverlay.XPos = tOverlay.XPos + OverViewLeft
tOverlay.YPos = tOverlay.YPos + OverViewTop

If tOverlay.RStyle = False Then
Angle = tOverlay.RotationAngle
X1 = tOverlay.XPos + RotateX(tOverlay.UL.X - tOverlay.UL.ScaleX, tOverlay.UL.Y - tOverlay.UL.ScaleY, Angle)
X2 = tOverlay.XPos + RotateX(tOverlay.UR.X + tOverlay.UR.ScaleX, tOverlay.UR.Y - tOverlay.UR.ScaleY, Angle)
X3 = tOverlay.XPos + RotateX(tOverlay.LL.X - tOverlay.LL.ScaleX, tOverlay.LL.Y + tOverlay.LL.ScaleY, Angle)
X4 = tOverlay.XPos + RotateX(tOverlay.LR.X + tOverlay.LR.ScaleX, tOverlay.LR.Y + tOverlay.LR.ScaleY, Angle)

Y1 = tOverlay.YPos + RotateY(tOverlay.UL.X - tOverlay.UL.ScaleX, tOverlay.UL.Y - tOverlay.UL.ScaleY, Angle)
Y2 = tOverlay.YPos + RotateY(tOverlay.UR.X + tOverlay.UR.ScaleX, tOverlay.UR.Y - tOverlay.UR.ScaleY, Angle)
Y3 = tOverlay.YPos + RotateY(tOverlay.LL.X - tOverlay.LL.ScaleX, tOverlay.LL.Y + tOverlay.LL.ScaleY, Angle)
Y4 = tOverlay.YPos + RotateY(tOverlay.LR.X + tOverlay.LR.ScaleX, tOverlay.LR.Y + tOverlay.LR.ScaleY, Angle)

TextureStrip(0) = MakeStrip(X1, Y1, 0, 1, tOverlay.C1, tOverlay.S1, tOverlay.Tx1, tOverlay.Ty1)
TextureStrip(1) = MakeStrip(X2, Y2, 0, 1, tOverlay.C2, tOverlay.S2, tOverlay.Tx2, tOverlay.Ty1)
TextureStrip(2) = MakeStrip(X3, Y3, 0, 1, tOverlay.C3, tOverlay.S3, tOverlay.Tx1, tOverlay.Ty2)
TextureStrip(3) = MakeStrip(X4, Y4, 0, 1, tOverlay.C4, tOverlay.S4, tOverlay.Tx2, tOverlay.Ty2)

'Set texture
            Direct3DDevice.SetTexture 0, Texture(tOverlay.TextureIndex)
            
'DrawStrip
            Direct3DDevice.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, TextureStrip(0), Len(TextureStrip(0))

Else:

With oSrcRect
    .Left = tOverlay.Get.X
    .Top = tOverlay.Get.Y
    .Right = .Left + tOverlay.Get.Width
    .Bottom = .Top + tOverlay.Get.Height
End With

With oTranslate
    .X = tOverlay.XPos + tOverlay.UL.X - tOverlay.UL.ScaleX
    .Y = tOverlay.YPos + tOverlay.UL.Y - tOverlay.UL.ScaleY
End With

With oRCenter
    .X = (tOverlay.UL.X - tOverlay.UL.ScaleX) * -1
    .Y = (tOverlay.UL.Y - tOverlay.UL.ScaleY) * -1
End With

X1 = oSrcRect.Right - oSrcRect.Left
X2 = (tOverlay.UR.X + tOverlay.UR.ScaleX) - (tOverlay.UL.X - tOverlay.UL.ScaleX)
Y1 = oSrcRect.Bottom - oSrcRect.Top
Y2 = (tOverlay.LL.Y + tOverlay.LL.ScaleY) - (tOverlay.UL.Y - tOverlay.LL.ScaleY)

With oScale
    .X = X2 / X1
    .Y = Y2 / Y1
End With

Angle = tOverlay.RotationAngle * (3.14 / 180)
Angle = Angle * -1

'draw sprite
oSprite.Begin
oSprite.Draw Texture(tOverlay.TextureIndex), oSrcRect, oScale, oRCenter, Angle, oTranslate, tOverlay.C1
oSprite.End

End If

'Exit for not Drawn

DontDraw:
End Sub

Private Sub RenderSprite(Order As Integer)
Dim tSprite As SpriteInfo
tSprite = Sprite(Order)

If tSprite.Void = True Then GoTo DontDraw

Dim X1 As Single, Y1 As Single
Dim X2 As Single, Y2 As Single
Dim X3 As Single, Y3 As Single
Dim X4 As Single, Y4 As Single
Dim Angle As Single

If tSprite.RStyle = False Then        'style 1

Sprite(Order).Drawn = True
Angle = tSprite.TempAngle
'offset the mapview left and top
tSprite.TempX = tSprite.TempX + MapView.Left
tSprite.TempY = tSprite.TempY + MapView.Top

X1 = tSprite.TempX + RotateX(tSprite.UL.X - tSprite.UL.ScaleX, tSprite.UL.Y - tSprite.UL.ScaleY, Angle)
X2 = tSprite.TempX + RotateX(tSprite.UR.X + tSprite.UR.ScaleX, tSprite.UR.Y - tSprite.UR.ScaleY, Angle)
X3 = tSprite.TempX + RotateX(tSprite.LL.X - tSprite.LL.ScaleX, tSprite.LL.Y + tSprite.LL.ScaleY, Angle)
X4 = tSprite.TempX + RotateX(tSprite.LR.X + tSprite.LR.ScaleX, tSprite.LR.Y + tSprite.LR.ScaleY, Angle)

Y1 = tSprite.TempY + RotateY(tSprite.UL.X - tSprite.UL.ScaleX, tSprite.UL.Y - tSprite.UL.ScaleY, Angle)
Y2 = tSprite.TempY + RotateY(tSprite.UR.X + tSprite.UR.ScaleX, tSprite.UR.Y - tSprite.UR.ScaleY, Angle)
Y3 = tSprite.TempY + RotateY(tSprite.LL.X - tSprite.LL.ScaleX, tSprite.LL.Y + tSprite.LL.ScaleY, Angle)
Y4 = tSprite.TempY + RotateY(tSprite.LR.X + tSprite.LR.ScaleX, tSprite.LR.Y + tSprite.LR.ScaleY, Angle)

Sprite(Order).DrawnX = tSprite.TempX
Sprite(Order).DrawnY = tSprite.TempY
If CheckIntersection(X1, Y1, X2, Y2, X3, Y3, X4, Y4) = False Then
    Sprite(Order).Drawn = False
End If

If Sprite(Order).Drawn = False Or tSprite.Visible = False Or tSprite.TRef = 0 Then GoTo DontDraw

TextureStrip(0) = MakeStrip(X1, Y1, 0, 1, tSprite.C1, tSprite.S1, tSprite.Tx1, tSprite.Ty1)
TextureStrip(1) = MakeStrip(X2, Y2, 0, 1, tSprite.C2, tSprite.S2, tSprite.Tx2, tSprite.Ty1)
TextureStrip(2) = MakeStrip(X3, Y3, 0, 1, tSprite.C3, tSprite.S3, tSprite.Tx1, tSprite.Ty2)
TextureStrip(3) = MakeStrip(X4, Y4, 0, 1, tSprite.C4, tSprite.S4, tSprite.Tx2, tSprite.Ty2)

'Set texture
            Direct3DDevice.SetTexture 0, Texture(tSprite.TRef)
            
'DrawStrip
            Direct3DDevice.DrawPrimitiveUP D3DPT_TRIANGLESTRIP, 2, TextureStrip(0), Len(TextureStrip(0))

Else:                       'style 2
Sprite(Order).Drawn = True
Angle = tSprite.TempAngle
'offset the mapview left and top
tSprite.TempX = tSprite.TempX + MapView.Left
tSprite.TempY = tSprite.TempY + MapView.Top

Sprite(Order).DrawnX = tSprite.TempX
Sprite(Order).DrawnY = tSprite.TempY
'check for clip, not as accurate as style1
If tSprite.TempX < MapView.Left - MapClip Then Sprite(Order).Drawn = False
If tSprite.TempX > MapView.Right + MapClip Then Sprite(Order).Drawn = False
If tSprite.TempY < MapView.Top - MapClip Then Sprite(Order).Drawn = False
If tSprite.TempY > MapView.Bottom + MapClip Then Sprite(Order).Drawn = False

If Sprite(Order).Drawn = False Or tSprite.Visible = False Or tSprite.TRef = 0 Then GoTo DontDraw

With oSrcRect
    .Left = tSprite.Get.X
    .Top = tSprite.Get.Y
    .Right = .Left + tSprite.Get.Width
    .Bottom = .Top + tSprite.Get.Height
End With

With oTranslate
    .X = tSprite.TempX + tSprite.UL.X - tSprite.UL.ScaleX
    .Y = tSprite.TempY + tSprite.UL.Y - tSprite.UL.ScaleY
End With

With oRCenter
    .X = (tSprite.UL.X - tSprite.UL.ScaleX) * -1
    .Y = (tSprite.UL.Y - tSprite.UL.ScaleY) * -1
End With

X1 = oSrcRect.Right - oSrcRect.Left
X2 = (tSprite.UR.X + tSprite.UR.ScaleX) - (tSprite.UL.X - tSprite.UL.ScaleX)
Y1 = oSrcRect.Bottom - oSrcRect.Top
Y2 = (tSprite.LL.Y + tSprite.LL.ScaleY) - (tSprite.UL.Y - tSprite.UL.ScaleY)

With oScale
    .X = X2 / X1
    .Y = Y2 / Y1
End With

Angle = Angle * (3.14 / 180) * -1

'draw sprite
oSprite.Begin
oSprite.Draw Texture(tSprite.TRef), oSrcRect, oScale, oRCenter, Angle, oTranslate, tSprite.C1
oSprite.End
End If

'Exit for not Drawn
DontDraw:
End Sub

Private Sub RenderText(Order As Integer)
    Dim Color As Long
    Dim tText As TextInfo
    Dim FI As Integer

    tText = TextEX(Order)
    Color = D3DColorARGB(tText.Color.Alpha, tText.Color.Red, tText.Color.Green, tText.Color.Blue)
    FI = tText.FIndex
    
    If tText.Visible = False Then Exit Sub
    
    With TextRect
        .Left = tText.XPos + TextViewLeft
        .Top = tText.YPos + TextViewTop
        .Right = .Left + tText.Width
        .Bottom = .Top + tText.Height
    End With
    
    D3DX.DrawText ScreenFont(FI), Color, tText.fText, TextRect, DT_TOP Or DT_LEFT
    
End Sub

Private Sub CheckAutoMoveSprite()
Dim Sref As Integer, Mref As Integer
Sref = AutoMove.Sref: Mref = Sprite(Sref).Mref

Dim X As Single, Y As Single
X = Sprite(Sref).XPos + Map(Mref).XPos
Y = Sprite(Sref).YPos + Map(Mref).YPos

Dim srect As RECT, CRect As RECT
With srect
    .Left = X - AutoMove.SWidth
    .Top = Y - AutoMove.SHeight
    .Right = .Left + (AutoMove.SWidth * 2)
    .Bottom = .Top + (AutoMove.SHeight * 2)
End With

X = MapView.Right - MapView.Left
X = X / 2
X = X - (AutoMove.Width / 2)
Y = MapView.Bottom - MapView.Top
Y = Y / 2
Y = Y - (AutoMove.Height / 2)

With CRect
    .Left = X
    .Top = Y
    .Right = .Left + AutoMove.Width
    .Bottom = .Top + AutoMove.Height
End With

    If srect.Left < CRect.Left And AutoMove.Left = True Then MoveMapRight Mref
    If srect.Right > CRect.Right And AutoMove.Right = True Then MoveMapLeft Mref
    If srect.Top < CRect.Top And AutoMove.Up = True Then MoveMapDown Mref
    If srect.Bottom > CRect.Bottom And AutoMove.Down = True Then MoveMapUp Mref

End Sub

Private Sub UpDateMapToLimit()
Dim XIndex As Single, YIndex As Single
Dim Mref As Integer, Sref As Integer
Dim Left As Single, Right As Single
Dim Top As Single, Bottom As Single
Dim XCount As Integer, YCount As Integer

'Find sub map that sprite is on
Sref = AutoMove.Sref
Mref = Sprite(Sref).Mref

XCount = Map(Mref).XCount
YCount = Map(Mref).YCount

XIndex = Sprite(Sref).XPos
YIndex = Sprite(Sref).YPos

XIndex = Int(XIndex / Map(Mref).SubMapWidth) + 1
YIndex = Int(YIndex / Map(Mref).SubMapHeight) + 1

Left = Map(Mref).XPos + ((XIndex - 1) * Map(Mref).SubMapWidth) + MapView.Left
Top = Map(Mref).YPos + ((YIndex - 1) * Map(Mref).SubMapHeight) + MapView.Top
Right = Left + Map(Mref).SubMapWidth
Bottom = Top + Map(Mref).SubMapHeight

If XIndex < 1 Then XIndex = XCount + XIndex
If XIndex > XCount Then XIndex = XIndex - XCount
If YIndex < 1 Then YIndex = YCount + YIndex
If YIndex > YCount Then YIndex = YIndex - YCount

AutoMove.Left = True
AutoMove.Right = True
AutoMove.Up = True
AutoMove.Down = True

If SubMap(Mref, XIndex, YIndex).LLeft = True And Left > MapView.Left Then
    MoveMapLeft Mref
    AutoMove.Left = False
End If
If SubMap(Mref, XIndex, YIndex).LRight = True And Right < MapView.Right Then
    MoveMapRight Mref
    AutoMove.Right = False
End If
If SubMap(Mref, XIndex, YIndex).LUp = True And Top > MapView.Top Then
    MoveMapUp Mref
    AutoMove.Up = False
End If
If SubMap(Mref, XIndex, YIndex).LDown = True And Bottom < MapView.Bottom Then
    MoveMapDown Mref
    AutoMove.Down = False
End If

End Sub

Private Sub CheckMapLimit()
Dim Index As Integer
Dim Diff As Single, Size As Single
    
    For Index = 1 To 8
    If Map(Index).Looping = False Then GoTo DontUpdate
        If Map(Index).XPos >= Map(Index).Width Then
            Size = Int(Map(Index).XPos / Map(Index).SubMapWidth)
            Diff = Map(Index).XPos - (Size * Map(Index).SubMapWidth)
            Map(Index).XPos = Diff
        End If
        If Map(Index).XPos <= -Map(Index).Width Then
            Size = Int(Abs(Map(Index).XPos) / Map(Index).SubMapWidth)
            Diff = Map(Index).XPos + (Size * Map(Index).SubMapWidth)
            Map(Index).XPos = Diff
        End If
        If Map(Index).YPos >= Map(Index).Height Then
            Size = Int(Map(Index).YPos / Map(Index).SubMapHeight)
            Diff = Map(Index).YPos - (Size * Map(Index).SubMapHeight)
            Map(Index).YPos = Diff
        End If
        If Map(Index).YPos <= -Map(Index).Height Then
            Size = Int(Abs(Map(Index).YPos) / Map(Index).SubMapHeight)
            Diff = Map(Index).YPos + (Size * Map(Index).SubMapHeight)
            Map(Index).YPos = Diff
        End If
DontUpdate:
    Next Index

End Sub

Private Sub CheckSpritePosition()
Dim Index As Integer
Dim CRect As RECT
Dim Ref As Variant
Dim X1 As Single, Y1 As Single
    If SpriteCount = 0 Then Exit Sub
    
    For Index = 1 To SpriteCount
        X1 = Sprite(Index).XPos
        Y1 = Sprite(Index).YPos
        Ref = Sprite(Index).Mref
        If Sprite(Index).Sref <> 0 Then GoTo DontOffset
        
    If Ref > 0 And Map(Ref).Looping = True Then
        
        With CRect
            .Left = -(Map(Ref).XPos) - MapClip
            .Top = -(Map(Ref).YPos) - MapClip
            .Right = .Left + Map(Ref).Width - 1
            .Bottom = .Top + Map(Ref).Height - 1
        End With
        
    'Check Farthest Right
        If X1 >= CRect.Left + Map(Ref).Width And X1 <= CRect.Right + Map(Ref).Width Then X1 = X1 - Map(Ref).Width
    'Check Farthest Left
        If X1 >= CRect.Left - Map(Ref).Width And X1 <= CRect.Right - Map(Ref).Width Then X1 = X1 + Map(Ref).Width
    'Check Farthest Down
        If Y1 >= CRect.Top + Map(Ref).Height And Y1 <= CRect.Bottom + Map(Ref).Height Then Y1 = Y1 - Map(Ref).Height
    'Check Farthest up
        If Y1 >= CRect.Top - Map(Ref).Height And Y1 <= CRect.Bottom - Map(Ref).Height Then Y1 = Y1 + Map(Ref).Height
    End If

DontOffset:
    Sprite(Index).TempX = X1 + Map(Ref).XPos
    Sprite(Index).TempY = Y1 + Map(Ref).YPos
    Sprite(Index).TempAngle = Sprite(Index).RAngle
    Sprite(Index).XPos = X1
    Sprite(Index).YPos = Y1
    Next Index
End Sub

Private Sub CheckSpriteReferences()
Dim Ref As Integer
Dim Index As Integer
Dim X As Single, Y As Single

If SpriteCount = 0 Then Exit Sub
For Index = 1 To SpriteCount
    Ref = Sprite(Index).Sref
    If Ref <> 0 Then
        X = RotateX(Sprite(Index).XPos, Sprite(Index).YPos, Sprite(Ref).TempAngle)
        Y = RotateY(Sprite(Index).XPos, Sprite(Index).YPos, Sprite(Ref).TempAngle)
        Sprite(Index).TempX = Sprite(Ref).TempX + X
        Sprite(Index).TempY = Sprite(Ref).TempY + Y
        If Sprite(Index).InheritAngle = True Then
            Sprite(Index).TempAngle = Sprite(Ref).TempAngle + Sprite(Index).RAngle
        End If
    End If
Next Index

End Sub

'Rotation Functions
Private Function RotateX(X As Single, Y As Single, Angle As Single) As Single
Dim Tx As Single, TAngle As Single
'turns the angle degrees into radians
TAngle = Angle * (3.14 / 180)

'Rotate on axis
Tx = X * Cos(TAngle) - Y * Sin(TAngle)

RotateX = Tx

End Function

Private Function RotateY(X As Single, Y As Single, Angle As Single) As Single
Dim Ty As Single, TAngle As Single
'turns the angle degrees into radians
TAngle = Angle * (3.14 / 180)

'Rotate on axis
Ty = Y * Cos(TAngle) + X * Sin(TAngle)

RotateY = Ty

End Function

Private Sub MoveMapUp(Index As Integer)
Dim T As Integer
    Map(Index).YPos = Map(Index).YPos - Map(Index).YInc
    For T = 1 To 8
        If Map(T).YRef = Index Then
            Map(T).YPos = Map(T).YPos - Map(T).YInc
        End If
    Next T
End Sub
Private Sub MoveMapDown(Index As Integer)
Dim T As Integer
    Map(Index).YPos = Map(Index).YPos + Map(Index).YInc
    For T = 1 To 8
        If Map(T).YRef = Index Then
            Map(T).YPos = Map(T).YPos + Map(T).YInc
        End If
    Next T
End Sub
Private Sub MoveMapLeft(Index As Integer)
Dim T As Integer
    Map(Index).XPos = Map(Index).XPos - Map(Index).XInc
    For T = 1 To 8
        If Map(T).XRef = Index Then
            Map(T).XPos = Map(T).XPos - Map(T).XInc
        End If
    Next T
End Sub
Private Sub MoveMapRight(Index As Integer)
Dim T As Integer
    Map(Index).XPos = Map(Index).XPos + Map(Index).XInc
    For T = 1 To 8
        If Map(T).XRef = Index Then
            Map(T).XPos = Map(T).XPos + Map(T).XInc
        End If
    Next T
End Sub

Private Function CheckIntersection(X1 As Single, Y1 As Single, X2 As Single, Y2 As Single, X3 As Single, Y3 As Single, X4 As Single, Y4 As Single) As Boolean
    Dim Check As Boolean
    Check = False
    If PointInMapView(X1, Y1) = True Then Check = True
    If PointInMapView(X2, Y2) = True Then Check = True
    If PointInMapView(X3, Y3) = True Then Check = True
    If PointInMapView(X4, Y4) = True Then Check = True
    CheckIntersection = Check
End Function

Private Function PointInMapView(X As Single, Y As Single) As Boolean
    If X >= MapView.Left - MapClip And X <= MapView.Right + MapClip And Y >= MapView.Top - MapClip And Y <= MapView.Bottom + MapClip Then
        PointInMapView = True
    Else:
        PointInMapView = False
    End If
End Function

Private Sub Class_Terminate()
    Set DBTexture = Nothing
    Set DBOverlay = Nothing
    Set DBMath = Nothing
    Set DBFPS = Nothing
    Set DBText = Nothing
    Set DBMap = Nothing
    Set DBSubMap = Nothing
    Set DBSprite = Nothing
    Set DBAutoMove = Nothing
    Set DBKeyInput = Nothing
    Set DBJoyStick = Nothing
    Set DBMusic = Nothing
    Set DBSound = Nothing
    Set DBMapData = Nothing
    Set DBCounter = Nothing
    Set DBResource = Nothing
    
    Set DirectX = Nothing
    Set Direct3D = Nothing
    Set D3DX = Nothing
    
    If DaBoodaKeyInputOn = True Then
        kDirectInputDevice.Unacquire
        Set kDirectInputDevice = Nothing
        Set DirectInput = Nothing
    End If

    If DaBoodaJoyStickOn = True Then
        JoyStickDevice.Unacquire
        Set JoyStickDevice = Nothing
        Set DirectInput = Nothing
    End If
    
    If DaBoodaMouseOn = True Then
        mDirectInputDevice.Unacquire
        Set mDirectInputDevice = Nothing
        Set DirectInput = Nothing
    End If
    
    If DaBoodaMusicOn = True Then
        DSControl.Stop
        Set DSAudio = Nothing
        Set DSEvent = Nothing
        Set DSControl = Nothing
        Set DSPosition = Nothing
    End If

    If DaBoodaSoundOn = True Then
        Set DirectSound = Nothing
        Set DirectSoundEnum = Nothing
    End If
    Dim T As Integer
    For T = 1 To 16
        Set ScreenFontDesc(T) = Nothing
        Set ScreenFont(T) = Nothing
    Next T
End Sub

Private Sub ReloadVertexData()

'Set Up Vertex Format
    Direct3DDevice.SetVertexShader FVF_VertexType
    
'Turn Off Lighting
Direct3DDevice.SetRenderState D3DRS_LIGHTING, False
Direct3DDevice.SetRenderState D3DRS_SPECULARENABLE, True

'Set Render State
    Direct3DDevice.SetRenderState D3DRS_SRCBLEND, D3DBLEND_SRCALPHA
    Direct3DDevice.SetRenderState D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA
    Direct3DDevice.SetRenderState D3DRS_ALPHABLENDENABLE, True
    
'Set Up Texture Stages
    Direct3DDevice.SetTextureStageState 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE
    Direct3DDevice.SetTextureStageState 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE
    Direct3DDevice.SetTextureStageState 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE

End Sub

Public Sub ClearScreen()
Dim T As Integer
'set default viewport
    Direct3DDevice.SetViewport DViewPort

For T = 1 To 2
    'doevents
    DoEvents
    
    'clear screen
    Direct3DDevice.Clear 0, ByVal 0, D3DCLEAR_TARGET, 0, 1#, 0

    'begin scene
    Direct3DDevice.BeginScene
    'end scene
    Direct3DDevice.EndScene
    'update fps
    DBFPS.UpdateFPS
    
    'present
    Direct3DDevice.Present ByVal 0, ByVal 0, 0, ByVal 0
    
Next T

'set up normal viewport
    Direct3DDevice.SetViewport ViewPort
    
End Sub

'The Counter Update Routine
Private Sub UpdateCounter()
Dim CIndex As Integer

'loop through all counters and update
For CIndex = 1 To CounterCount
    'check to see if counter is even on
    If Counter(CIndex).On = False Then GoTo SkipCount
    'do a with to clarify
    With Counter(CIndex)
    'check to see if loops are not reached
    If .TotalLoops <> .MaxLoops Then
    'reset the counter done
    .CounterDone = False
    'select the case for current loop
    Select Case .CLoop
        '1 loop method
        Case 1
        'increment timer
        .Timer = .Timer + 1
        'check timer and do update if reached
        If .Timer = .Timer1 Then
        'reset timer
        .Timer = 0
        'increment value if value is not equal to max1
        If .Value <> .Max1 Then
            If .Value < .Max1 Then .Value = .Value + .Increment1
            If .Value > .Max1 Then .Value = .Value - .Increment1
        Else:       'Reset or loop or move to next loop if value is equal
        'do 1 loop method
            If .MLoop = 1 Then
                .Value = .Min1
                .TotalLoops = .TotalLoops + 1
            Else:       'the only other loop method is 2
                'set up to run second loop on next pass
                .CLoop = 2
                .Value = .Min2
            End If
        
        End If
        
        End If
        'do second loop if cloop = 2
        Case 2      'this loop is real similar to the first one, just different variables
        .Timer = .Timer + 1
        If .Timer = .Timer2 Then
        .Timer = 0
        
        If .Value <> .Max2 Then
            If .Value < .Max2 Then .Value = .Value + .Increment2
            If .Value > .Max2 Then .Value = .Value - .Increment2
        Else:
                .CLoop = 1
                .Value = .Min1
                .TotalLoops = .TotalLoops + 1
        End If
        
        End If
        
    End Select
        
    End If
    
If .Looping = True And .TotalLoops = .MaxLoops Then
    .CounterDone = True
    .TotalLoops = 0
End If

If .Looping = False And .TotalLoops = .MaxLoops Then
    .CounterDone = True
    If .AutoOff = True Then
        .On = False
    End If
    If .MLoop = 1 Then
        .Value = .Max1
    Else:
        .Value = .Max2
    End If
End If

End With

SkipCount:
Next CIndex
End Sub

'check for windowed
Private Sub CheckWindowed()
    'check for minimized first
    If WindowObject.WindowState = vbMinimized Then
        'pause music
        If DaBoodaMusicOn = True Then
            DBMusic.Pause
        End If
        'unaquire mouse
        If DaBoodaMouseOn = True Then
            mDirectInputDevice.Unacquire
        End If
        'do a loop
        Do While WindowObject.WindowState = vbMinimized
            'do events
            DoEvents
        Loop
        'unpause music
        If DaBoodaMusicOn = True Then
            DBMusic.UnPause
        End If
        'aquire mouse
        If DaBoodaMouseOn = True Then
            mDirectInputDevice.Acquire
        End If
    End If
    'Check for lost focus
    If (GetActiveWindow() <> WindowObject.hwnd) Then
        'pause music
        If DaBoodaMusicOn = True Then
            DBMusic.Pause
        End If
        'unaquire mouse
        If DaBoodaMouseOn = True Then
            mDirectInputDevice.Unacquire
        End If
        'do a loop
        Do While (GetActiveWindow() <> WindowObject.hwnd)
            'do events
            DoEvents
        Loop
        'unpause music
        If DaBoodaMusicOn = True Then
            DBMusic.UnPause
        End If
        'aquire mouse
        If DaBoodaMouseOn = True Then
            mDirectInputDevice.Acquire
        End If
    End If
End Sub
