VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DBVertexGroup"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit
'v1.4 addition
'this class allows for one to add a group of veretexes to the
'game, that they can then manipulate in 3d
'also allows the attachment of sprites to manipulate
'since this is only a reference, it does not need to be part of the engine,
'so multiple instances can be created

'type for 3dvertex
Private Type TripleVertex
    X As Single
    Y As Single
    Z As Single
End Type

'type for color for fog
Private Type SColor
    Red As Integer
    Green As Integer
    Blue As Integer
    Alpha As Integer
End Type

'vertex info
Private Vertex() As TripleVertex

'temp vertex info
Private TVertex() As TripleVertex

'vertex count
Private VertexCount As Integer

'origins
Private XOrigin As Single
Private YOrigin As Single

'variables for sprites
Private ASprite() As Integer
Private AColor() As SColor
Private tSprite() As Integer
Private TColor() As SColor
Private oSpriteA() As Integer
Private oSpriteB() As Integer
Private SwapCount As Integer

'variables for sprite Scales
Private ScaleFactor As Single
Private ScaleMin As Single
Private ScaleMax As Single

'Variables for sprite Fog
Private FogStart As Single
Private FogEnd As Single
Private FogRed As Boolean
Private FogGreen As Boolean
Private FogBlue As Boolean
Private FogAlpha As Boolean
Private Range As Single

'other varibles
Private VLoop As Integer

Public Sub VertexAdd(sX As Single, sY As Single, sZ As Single)
VertexCount = VertexCount + 1
ReDim Preserve Vertex(VertexCount) As TripleVertex
ReDim Preserve TVertex(VertexCount) As TripleVertex
ReDim Preserve ASprite(VertexCount) As Integer
ReDim Preserve AColor(VertexCount) As SColor
ReDim Preserve tSprite(VertexCount) As Integer
ReDim Preserve TColor(VertexCount) As SColor

With Vertex(VertexCount)
    .X = sX
    .Y = sY
    .Z = sZ
End With

End Sub

Public Sub VertexSet(Index As Integer, sX As Single, sY As Single, sZ As Single)
With Vertex(Index)
    .X = sX
    .Y = sY
    .Z = sZ
End With
End Sub

Public Sub VertexClear()
ReDim Vertex(0) As TripleVertex
ReDim TVertex(0) As TripleVertex
ReDim ASprite(0) As Integer
ReDim AColor(0) As SColor
ReDim tSprite(0) As Integer
ReDim TColor(0) As SColor
ReDim oSpriteA(0) As Integer
ReDim oSpriteB(0) As Integer
VertexCount = 0
End Sub

Public Sub VertexReset()
For VLoop = 1 To VertexCount
    TVertex(VLoop) = Vertex(VLoop)
    tSprite(VLoop) = ASprite(VLoop)
    TColor(VLoop) = AColor(VLoop)
Next VLoop
If SwapCount > 0 Then
    For VLoop = SwapCount To 1 Step -1
        Sprite(0) = Sprite(oSpriteA(VLoop))
        Sprite(oSpriteA(VLoop)) = Sprite(oSpriteB(VLoop))
        Sprite(oSpriteB(VLoop)) = Sprite(0)
    Next VLoop
End If

End Sub

Public Function VertexGetCount() As Integer
    VertexGetCount = VertexCount
End Function

Public Sub VertexRotateX(Xangle As Single)
Dim Xn As Single, Tx As Single, Ty As Single, TZ As Single
Xn = Xangle * (3.141592654 / 180)

    For VLoop = 1 To VertexCount
        With TVertex(VLoop)
        'rotate on x axis
            Tx = .X: Ty = .Y: TZ = .Z
            .Y = Ty * Cos(Xn) - TZ * Sin(Xn)
            .Z = Ty * Sin(Xn) + TZ * Cos(Xn)
            .X = Tx
        End With
    Next VLoop
    
End Sub

Public Sub VertexRotateY(Yangle As Single)
Dim Yn As Single, Tx As Single, Ty As Single, TZ As Single
Yn = Yangle * (3.141592654 / 180)

    For VLoop = 1 To VertexCount
        With TVertex(VLoop)
        'rotate on x axis
            Tx = .X: Ty = .Y: TZ = .Z
            .Z = TZ * Cos(Yn) - Tx * Sin(Yn)
            .X = TZ * Sin(Yn) + Tx * Cos(Yn)
            .Y = Ty
        End With
    Next VLoop
    
End Sub

Public Sub VertexRotateZ(Zangle As Single)
Dim Zn As Single, Tx As Single, Ty As Single, TZ As Single
Zn = Zangle * (3.141592654 / 180)

    For VLoop = 1 To VertexCount
        With TVertex(VLoop)
        'rotate on x axis
            Tx = .X: Ty = .Y: TZ = .Z
            .X = Tx * Cos(Zn) - Ty * Sin(Zn)
            .Y = Ty * Cos(Zn) + Tx * Sin(Zn)
            .Z = TZ
        End With
    Next VLoop
    
End Sub

Public Sub VertexScaleX(ScaleX As Single)
    For VLoop = 1 To VertexCount
        With TVertex(VLoop)
        'scale on x
        .X = ReturnPercentage(.X, ScaleX)
        End With
    Next VLoop
End Sub

Public Sub VertexScaleY(ScaleY As Single)
    For VLoop = 1 To VertexCount
        With TVertex(VLoop)
        'scale on y
        .Y = ReturnPercentage(.Y, ScaleY)
        End With
    Next VLoop
End Sub

Public Sub VertexScaleZ(ScaleZ As Single)
    For VLoop = 1 To VertexCount
        With TVertex(VLoop)
        'scale on z
        .Z = ReturnPercentage(.Z, ScaleZ)
        End With
    Next VLoop
End Sub

Public Sub VertexScale(ScaleAll As Single)
    For VLoop = 1 To VertexCount
        'scale on x
        TVertex(VLoop).X = ReturnPercentage(TVertex(VLoop).X, ScaleAll)
        'scale on y
        TVertex(VLoop).Y = ReturnPercentage(TVertex(VLoop).Y, ScaleAll)
        'scale on z
        TVertex(VLoop).Z = ReturnPercentage(TVertex(VLoop).Z, ScaleAll)
    Next VLoop
End Sub

Public Sub VertexPerspective()
Dim Tx As Single, Ty As Single, TZ As Single

For VLoop = 1 To VertexCount
    With TVertex(VLoop)
        Tx = .X: Ty = .Y: TZ = .Z
        If TZ < -999 Then TZ = -999
        If TZ > 999 Then TZ = 999
        TZ = TZ + 1000
        TZ = 2000 - TZ
        Tx = (Tx / TZ) * 1000
        Ty = (Ty / TZ) * 1000
        .X = Tx
        .Y = Ty
    End With
Next VLoop
End Sub

Public Sub VertexTranslate(X As Single, Y As Single, Z As Single)
For VLoop = 1 To VertexCount
    With TVertex(VLoop)
        .X = .X + X
        .Y = .Y + Y
        .Z = .Z + Z
    End With
Next VLoop
End Sub

Public Sub VertexSetOrigin(X As Single, Y As Single)
    XOrigin = X
    YOrigin = Y
End Sub

Public Sub VertexUpdateOrigin()
For VLoop = 1 To VertexCount
    TVertex(VLoop).X = TVertex(VLoop).X + XOrigin
    TVertex(VLoop).Y = TVertex(VLoop).Y + YOrigin
Next VLoop
End Sub

Public Sub SpriteAttach(VIndex As Integer, SIndex As Integer)
    ASprite(VIndex) = SIndex
    With AColor(VIndex)
        .Red = Sprite(SIndex).UL.CRed
        .Green = Sprite(SIndex).UL.CGreen
        .Blue = Sprite(SIndex).UL.CBlue
        .Alpha = Sprite(SIndex).UL.Alpha
    End With
End Sub

Public Sub SpriteAttachList(SStart As Integer)
Dim Index As Integer
Index = SStart
    For VLoop = 1 To VertexCount
        ASprite(VLoop) = Index
        With AColor(VLoop)
        .Red = Sprite(Index).UL.CRed
        .Green = Sprite(Index).UL.CGreen
        .Blue = Sprite(Index).UL.CBlue
        .Alpha = Sprite(Index).UL.Alpha
        End With
        Index = Index + 1
    Next VLoop
End Sub

Public Sub SpriteSetUpFog(FStart As Single, FEnd As Single, Optional Red As Boolean = True, Optional Green As Boolean = True, Optional Blue As Boolean = True, Optional Alpha As Boolean = False)
    FogStart = FStart
    FogEnd = FEnd
    FogRed = Red
    FogGreen = Green
    FogBlue = Blue
    FogAlpha = Alpha
    Range = 0
If FogStart < 0 And FogEnd < 0 Then
    Range = Abs(FogEnd) - Abs(FogStart)
ElseIf FogStart > 0 And FogEnd < 0 Then
    Range = FogStart + Abs(FogEnd)
ElseIf FogStart > 0 And FogEnd > 0 Then
    Range = FogStart - FogEnd
ElseIf FogStart = 0 And FogEnd < 0 Then
    Range = Abs(FogEnd)
ElseIf FogStart < FogEnd Then
    Range = 0
End If

End Sub

Public Sub SpriteSortZ()
Dim Swap As Boolean
SwapCount = 0
ZStart:
Swap = False
    For VLoop = 1 To VertexCount - 1
      If TVertex(VLoop).Z <= TVertex(VLoop + 1).Z Then GoTo ZPass
      'sort it other wise
      'sort sprite first
      Sprite(0) = Sprite(tSprite(VLoop))
      Sprite(tSprite(VLoop)) = Sprite(tSprite(VLoop + 1))
      Sprite(tSprite(VLoop + 1)) = Sprite(0)
      'sort vertexs
      TVertex(0) = TVertex(VLoop)
      TVertex(VLoop) = TVertex(VLoop + 1)
      TVertex(VLoop + 1) = TVertex(0)
      'sort order, to keep things straight
      SwapCount = SwapCount + 1
      ReDim Preserve oSpriteA(SwapCount) As Integer
      ReDim Preserve oSpriteB(SwapCount) As Integer
      oSpriteA(SwapCount) = tSprite(VLoop)
      oSpriteB(SwapCount) = tSprite(VLoop + 1)
      Swap = True
ZPass:
     Next VLoop
'check to see if swap occured
    If Swap = True Then GoTo ZStart

End Sub

Public Sub SpriteUpdateXY()
    For VLoop = 1 To VertexCount
        With Sprite(tSprite(VLoop))
            .XPos = XOrigin + TVertex(VLoop).X
            .YPos = YOrigin + TVertex(VLoop).Y
        End With
    Next VLoop
End Sub

Public Sub SpriteUpdateScale()
Dim Dir As Single
    For VLoop = 1 To VertexCount
        If TVertex(VLoop).Z < 0 Then
            Dir = -1
        Else:
            Dir = 1
        End If
        With Sprite(tSprite(VLoop))
            .UL.ScaleX = Abs(ReturnPerspective(TVertex(VLoop).Z, .UL.X) - .UL.X) * Dir
            .UR.ScaleX = Abs(ReturnPerspective(TVertex(VLoop).Z, .UR.X) - .UR.X) * Dir
            .LL.ScaleX = Abs(ReturnPerspective(TVertex(VLoop).Z, .LL.X) - .LL.X) * Dir
            .LR.ScaleX = Abs(ReturnPerspective(TVertex(VLoop).Z, .LR.X) - .LR.X) * Dir
            .UL.ScaleY = Abs(ReturnPerspective(TVertex(VLoop).Z, .UL.Y) - .UL.Y) * Dir
            .UR.ScaleY = Abs(ReturnPerspective(TVertex(VLoop).Z, .UR.Y) - .UR.Y) * Dir
            .LL.ScaleY = Abs(ReturnPerspective(TVertex(VLoop).Z, .LL.Y) - .LL.Y) * Dir
            .LR.ScaleY = Abs(ReturnPerspective(TVertex(VLoop).Z, .LR.Y) - .LR.Y) * Dir
        End With
    Next VLoop

End Sub

Public Sub SpriteUpdateFog()
Dim TCol As Integer

For VLoop = 1 To VertexCount
    'do red first
    If FogRed = True Then
        TCol = FindZColor(TVertex(VLoop).Z, TColor(VLoop).Red)
        Sprite(tSprite(VLoop)).UL.CRed = TCol
        Sprite(tSprite(VLoop)).UR.CRed = TCol
        Sprite(tSprite(VLoop)).LL.CRed = TCol
        Sprite(tSprite(VLoop)).LR.CRed = TCol
    End If
    'do green first
    If FogGreen = True Then
        TCol = FindZColor(TVertex(VLoop).Z, TColor(VLoop).Green)
        Sprite(tSprite(VLoop)).UL.CGreen = TCol
        Sprite(tSprite(VLoop)).UR.CGreen = TCol
        Sprite(tSprite(VLoop)).LL.CGreen = TCol
        Sprite(tSprite(VLoop)).LR.CGreen = TCol
    End If
    'do Blue first
    If FogBlue = True Then
        TCol = FindZColor(TVertex(VLoop).Z, TColor(VLoop).Blue)
        Sprite(tSprite(VLoop)).UL.CBlue = TCol
        Sprite(tSprite(VLoop)).UR.CBlue = TCol
        Sprite(tSprite(VLoop)).LL.CBlue = TCol
        Sprite(tSprite(VLoop)).LR.CBlue = TCol
    End If
    'do Alpha first
    If FogAlpha = True Then
        TCol = FindZColor(TVertex(VLoop).Z, TColor(VLoop).Alpha)
        Sprite(tSprite(VLoop)).UL.Alpha = TCol
        Sprite(tSprite(VLoop)).UR.Alpha = TCol
        Sprite(tSprite(VLoop)).LL.Alpha = TCol
        Sprite(tSprite(VLoop)).LR.Alpha = TCol
    End If
    With Sprite(tSprite(VLoop))
        .C1 = D3DColorARGB(.UL.Alpha, .UL.CRed, .UL.CGreen, .UL.CBlue)
        .C2 = D3DColorARGB(.UR.Alpha, .UR.CRed, .UR.CGreen, .UR.CBlue)
        .C3 = D3DColorARGB(.LL.Alpha, .LL.CRed, .LL.CGreen, .LL.CBlue)
        .C4 = D3DColorARGB(.LR.Alpha, .LR.CRed, .LR.CGreen, .LR.CBlue)
    End With
Next VLoop
End Sub

Public Sub CreatePolygon(SIndex As Integer, UL As Integer, UR As Integer, LL As Integer, LR As Integer, Optional Fog As Boolean = True)
With Sprite(SIndex)
    .UL.X = TVertex(UL).X
    .UL.Y = TVertex(UL).Y
    .UR.X = TVertex(UR).X
    .UR.Y = TVertex(UR).Y
    .LL.X = TVertex(LL).X
    .LL.Y = TVertex(LL).Y
    .LR.X = TVertex(LR).X
    .LR.Y = TVertex(LR).Y
    If Fog = True Then
        If FogAlpha = True Then
            .UL.Alpha = FindZColor(TVertex(UL).Z, 255)
            .UR.Alpha = FindZColor(TVertex(UR).Z, 255)
            .LL.Alpha = FindZColor(TVertex(LL).Z, 255)
            .LR.Alpha = FindZColor(TVertex(LR).Z, 255)
        End If
        If FogRed = True Then
            .UL.CRed = FindZColor(TVertex(UL).Z, 255)
            .UR.CRed = FindZColor(TVertex(UR).Z, 255)
            .LL.CRed = FindZColor(TVertex(LL).Z, 255)
            .LR.CRed = FindZColor(TVertex(LR).Z, 255)
        End If
        If FogGreen = True Then
            .UL.CGreen = FindZColor(TVertex(UL).Z, 255)
            .UR.CGreen = FindZColor(TVertex(UR).Z, 255)
            .LL.CGreen = FindZColor(TVertex(LL).Z, 255)
            .LR.CGreen = FindZColor(TVertex(LR).Z, 255)
        End If
        If FogBlue = True Then
            .UL.CBlue = FindZColor(TVertex(UL).Z, 255)
            .UR.CBlue = FindZColor(TVertex(UR).Z, 255)
            .LL.CBlue = FindZColor(TVertex(LL).Z, 255)
            .LR.CBlue = FindZColor(TVertex(LR).Z, 255)
        End If
        .C1 = D3DColorARGB(.UL.Alpha, .UL.CRed, .UL.CGreen, .UL.CBlue)
        .C2 = D3DColorARGB(.UR.Alpha, .UR.CRed, .UR.CGreen, .UR.CBlue)
        .C3 = D3DColorARGB(.LL.Alpha, .LL.CRed, .LL.CGreen, .LL.CBlue)
        .C4 = D3DColorARGB(.LR.Alpha, .LR.CRed, .LR.CGreen, .LR.CBlue)
    End If
End With

End Sub

Private Function FindZColor(TZ As Single, Col As Integer) As Integer
Dim val As Single, Diff As Single
Diff = Col / Range

'do fog if col is in range
If TZ <= FogStart And TZ >= FogEnd Then
    val = Abs(FogStart - TZ)
    val = val * Diff
End If


val = Col - val

If TZ < FogEnd Then val = 0
If TZ > FogStart Then val = Col

FindZColor = Int(val)

End Function

Public Function ReturnX(VIndex As Integer) As Single
    ReturnX = TVertex(VIndex).X
End Function
Public Function ReturnY(VIndex As Integer) As Single
    ReturnY = TVertex(VIndex).Y
End Function
Public Function ReturnZ(VIndex As Integer) As Single
    ReturnZ = TVertex(VIndex).Z
End Function

Private Function ReturnPercentage(Number As Single, Percent As Single) As Single
    ReturnPercentage = Number * (Percent * 0.01)
End Function

Private Function ReturnPerspective(Z As Single, Value As Single) As Single
Dim TZ As Single
    TZ = Z
        If TZ < -999 Then TZ = -999
        If TZ > 999 Then TZ = 999
        TZ = TZ + 1000
        TZ = 2000 - TZ
        ReturnPerspective = (Value / TZ) * 1000
End Function

Private Sub Class_Initialize()
ReDim Vertex(0) As TripleVertex
ReDim TVertex(0) As TripleVertex
ReDim ASprite(0) As Integer
ReDim AColor(0) As SColor
ReDim tSprite(0) As Integer
ReDim TColor(0) As SColor
ReDim oSpriteA(0) As Integer
ReDim oSpriteB(0) As Integer
End Sub

Private Sub Class_Terminate()
ReDim Vertex(0) As TripleVertex
ReDim TVertex(0) As TripleVertex
ReDim ASprite(0) As Integer
ReDim AColor(0) As SColor
ReDim tSprite(0) As Integer
ReDim TColor(0) As SColor
ReDim oSpriteA(0) As Integer
ReDim oSpriteB(0) As Integer
End Sub
